/*******************************************************************************
 * Copyright (c) 2004, 2007 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.datatools.enablement.oda.xml.util;

import java.io.IOException;
import java.io.InputStream;

/**
 * MemoryCacheDataSource represents the stream which can be cached in memory,
 * need not use the disk
 * 
 */
class MemoryCacheDataSource implements IInputXMLDataSource
{

	byte[] byteArray;
	boolean overReadLimit = false;
	int currentPosition = -1, length;

	/**
	 * 
	 * @param stream
	 * @param size
	 */
	MemoryCacheDataSource( InputStream stream, long size )
	{
		byteArray = new byte[(int) size];
		int ch = -1;
		try
		{
			int i = 0;
			for ( ; i < size && ( ch = stream.read( ) ) != -1; i++ )
			{
				byteArray[i] = (byte) ch;
			}
			length = i;
			if ( length < size )
			{
				byteArray[i] = -1;
			}
		}
		catch ( IOException e )
		{
		}
		if ( ch != -1 )
			overReadLimit = true;
	}

	/**
	 * 
	 * @return
	 */
	boolean canUseMemoryCache( )
	{
		return !this.overReadLimit;
	}
	
	/*
	 * @see org.eclipse.datatools.enablement.oda.xml.util.IInputStream#read()
	 */
	public int read( )
	{
		currentPosition++;
		if ( currentPosition < length )
			return byteArray[currentPosition];
		else
			return -1;
	}

	/*
	 * @see org.eclipse.datatools.enablement.oda.xml.util.IInputStream#init()
	 */
	public void init( )
	{
		this.currentPosition = -1;
	}

	/*
	 * @see org.eclipse.datatools.enablement.oda.xml.util.IInputStream#close()
	 */
	public void close( ) throws IOException
	{
	}
}
