/*******************************************************************************
 * Copyright  2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.datatools.sqltools.sqlbuilder.views.select;

import java.util.List;

import org.eclipse.datatools.modelbase.sql.query.QueryExpressionRoot;
import org.eclipse.datatools.modelbase.sql.query.QuerySelect;
import org.eclipse.datatools.modelbase.sql.query.QuerySelectStatement;
import org.eclipse.datatools.sqltools.sqlbuilder.Messages;
import org.eclipse.datatools.sqltools.sqlbuilder.SQLBuilderContextIds;
import org.eclipse.datatools.sqltools.sqlbuilder.model.SQLDomainModel;
import org.eclipse.datatools.sqltools.sqlbuilder.model.SelectHelper;
import org.eclipse.datatools.sqltools.sqlbuilder.util.ViewUtility;
import org.eclipse.datatools.sqltools.sqlbuilder.views.StatementNameViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.help.WorkbenchHelp;


/**
 * Create the viewer for the statement name
 */
public class SelectStatementNameViewer extends StatementNameViewer implements Listener, SelectionListener {

    private Button checkBox;

    public SelectStatementNameViewer(SQLDomainModel sqlDomainModel) {
        super(sqlDomainModel);
        setContentProvider(sqlDomainModel.createContentProvider());
    }

    public void setInput(Object input) {
        setDistinct(input);
        super.setInput(input);
    }

    public void refresh() {
        setDistinct(getInput());
        super.refresh();
    }

    public void setDistinct(Object input) {
        if (input instanceof QuerySelectStatement || input instanceof QuerySelect) {
            QuerySelect qSelect = null;
            if (input instanceof QuerySelectStatement) {
                QuerySelectStatement select = (QuerySelectStatement) input;
                qSelect = SelectHelper.getQuerySelect(select);
            }
            else {
                qSelect = (QuerySelect) input;
            }
            if (qSelect != null && qSelect.isDistinct() == true) {
                checkBox.setSelection(true);
            }
            else {
                checkBox.setSelection(false);
            }
        }
    }

    public Control createControl(Composite parent) {
        // Create the main UI container
        mainUIComponent = new Composite(parent, SWT.NULL);

        GridLayout layout = new GridLayout();

        layout.marginHeight = 5;
        layout.marginWidth = 5;
        layout.horizontalSpacing = 5;
        layout.verticalSpacing = 5;

        layout.numColumns = 3;
        mainUIComponent.setLayout(layout);

        mainUIComponent.setLayoutData(ViewUtility.createHorizontalFill());

        ViewUtility.createLabel(mainUIComponent, Messages._UI_LABEL_STATEMENT);
        nameField = new Text(mainUIComponent, SWT.SINGLE | SWT.BORDER | SWT.READ_ONLY);
        
        // WITH statements use this viewer as well, set the help accordingly
        if(currentStatement instanceof QuerySelectStatement)
        {
            QuerySelectStatement sqlSelectStatement = (QuerySelectStatement)currentStatement;
            QueryExpressionRoot expr = sqlSelectStatement.getQueryExpr();
            if(expr != null)
            {
                List withClause = expr.getWithClause();
                if(! withClause.isEmpty()){
                    WorkbenchHelp.setHelp(nameField, SQLBuilderContextIds.SQDW_STMT_NAME_FIELD);
                }
            }
            else
            {
                WorkbenchHelp.setHelp(nameField, SQLBuilderContextIds.SQLB_SELECT_VIEW);
            }
        }
        //nameField.setFont(ViewUtility.getFont());
        GridData data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.grabExcessHorizontalSpace = true;
        data.widthHint = 20;
        nameField.setLayoutData(data);

        checkBox = ViewUtility.createCheckBox(mainUIComponent, "DISTINCT"); //$NON-NLS-1$
        checkBox.addSelectionListener(this);
        WorkbenchHelp.setHelp(checkBox, SQLBuilderContextIds.SQLB_SELECT_VIEW);

        hookControl(getControl());
        return getControl();
    }

    public Control getControl() {
        return mainUIComponent;
    }

    public void widgetSelected(SelectionEvent event) {
        if (event.widget != null && event.widget == checkBox) {
            QuerySelect qSelect = SelectHelper.getQuerySelect((QuerySelectStatement) currentStatement);
            if (qSelect != null) {
                qSelect.setDistinct(checkBox.getSelection());
                SelectHelper.refresh((QuerySelectStatement) currentStatement);
            }
        }
    }

    public void widgetDefaultSelected(SelectionEvent event) {
    }

    public void setEnabled(boolean enable) {
        checkBox.setEnabled(enable);
    }
}